package javax.swing.plaf.metal;

import javax.swing.Icon;
import java.awt.Dimension;
import java.awt.Insets;
import javax.swing.plaf.UIResource;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.UIManager;
import java.awt.LayoutManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Color;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

public class MetalTabbedPaneUI extends BasicTabbedPaneUI
{
    protected int minTabWidth;
    private Color unselectedBackground;
    protected Color tabAreaBackground;
    protected Color selectColor;
    protected Color selectHighlight;
    private boolean tabsOpaque;
    private boolean ocean;
    private Color oceanSelectedBorderColor;
    
    public MetalTabbedPaneUI() {
        this.minTabWidth = 40;
        this.tabsOpaque = true;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MetalTabbedPaneUI();
    }
    
    @Override
    protected LayoutManager createLayoutManager() {
        if (this.tabPane.getTabLayoutPolicy() == 1) {
            return super.createLayoutManager();
        }
        return new MetalTabbedPaneUI.TabbedPaneLayout();
    }
    
    @Override
    protected void installDefaults() {
        super.installDefaults();
        this.tabAreaBackground = UIManager.getColor("TabbedPane.tabAreaBackground");
        this.selectColor = UIManager.getColor("TabbedPane.selected");
        this.selectHighlight = UIManager.getColor("TabbedPane.selectHighlight");
        this.tabsOpaque = UIManager.getBoolean("TabbedPane.tabsOpaque");
        this.unselectedBackground = UIManager.getColor("TabbedPane.unselectedBackground");
        this.ocean = MetalLookAndFeel.usingOcean();
        if (this.ocean) {
            this.oceanSelectedBorderColor = UIManager.getColor("TabbedPane.borderHightlightColor");
        }
    }
    
    @Override
    protected void paintTabBorder(Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
        int n7 = n4 + (n6 - 1);
        int n8 = n3 + (n5 - 1);
        switch (n) {
            case 2: {
                this.paintLeftTabBorder(n2, graphics, n3, n4, n5, n6, n7, n8, b);
                break;
            }
            case 3: {
                this.paintBottomTabBorder(n2, graphics, n3, n4, n5, n6, n7, n8, b);
                break;
            }
            case 4: {
                this.paintRightTabBorder(n2, graphics, n3, n4, n5, n6, n7, n8, b);
                break;
            }
            default: {
                this.paintTopTabBorder(n2, graphics, n3, n4, n5, n6, n7, n8, b);
                break;
            }
        }
    }
    
    protected void paintTopTabBorder(int tabIndex, Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
        int runForTab = this.getRunForTab(this.tabPane.getTabCount(), tabIndex);
        int lastTabInRun = this.lastTabInRun(this.tabPane.getTabCount(), runForTab);
        int n7 = this.tabRuns[runForTab];
        int leftToRight = MetalUtils.isLeftToRight(this.tabPane);
        int selectedIndex = this.tabPane.getSelectedIndex();
        int n8 = n4 - 1;
        int n9 = n3 - 1;
        if (this.shouldFillGap(runForTab, tabIndex, n, n2)) {
            graphics.translate(n, n2);
            if (!(leftToRight == 0)) {
                graphics.setColor(this.getColorForGap(runForTab, n, n2 + 1));
                graphics.fillRect(1, 0, 5, 3);
                graphics.fillRect(1, 3, 2, 2);
            }
            else {
                graphics.setColor(this.getColorForGap(runForTab, n + n3 - 1, n2 + 1));
                graphics.fillRect(n9 - 5, 0, 5, 3);
                graphics.fillRect(n9 - 2, 3, 2, 2);
            }
            graphics.translate(-n, -n2);
        }
        graphics.translate(n, n2);
        if (this.ocean && b) {
            graphics.setColor(this.oceanSelectedBorderColor);
        }
        else {
            graphics.setColor(this.darkShadow);
        }
        if (!(leftToRight == 0)) {
            graphics.drawLine(1, 5, 6, 0);
            graphics.drawLine(6, 0, n9, 0);
            if (tabIndex == lastTabInRun) {
                graphics.drawLine(n9, 1, n9, n8);
            }
            if (this.ocean && tabIndex - 1 == selectedIndex && runForTab == this.getRunForTab(this.tabPane.getTabCount(), selectedIndex)) {
                graphics.setColor(this.oceanSelectedBorderColor);
            }
            if (tabIndex != this.tabRuns[this.runCount - 1]) {
                if (this.ocean && b) {
                    graphics.drawLine(0, 6, 0, n8);
                    graphics.setColor(this.darkShadow);
                    graphics.drawLine(0, 0, 0, 5);
                }
                else {
                    graphics.drawLine(0, 0, 0, n8);
                }
            }
            else {
                graphics.drawLine(0, 6, 0, n8);
            }
        }
        else {
            graphics.drawLine(n9 - 1, 5, n9 - 6, 0);
            graphics.drawLine(n9 - 6, 0, 0, 0);
            if (tabIndex == lastTabInRun) {
                graphics.drawLine(0, 1, 0, n8);
            }
            if (this.ocean && tabIndex - 1 == selectedIndex && runForTab == this.getRunForTab(this.tabPane.getTabCount(), selectedIndex)) {
                graphics.setColor(this.oceanSelectedBorderColor);
                graphics.drawLine(n9, 0, n9, n8);
            }
            else if (this.ocean && b) {
                graphics.drawLine(n9, 6, n9, n8);
                if (tabIndex != 0) {
                    graphics.setColor(this.darkShadow);
                    graphics.drawLine(n9, 0, n9, 5);
                }
            }
            else if (tabIndex != this.tabRuns[this.runCount - 1]) {
                graphics.drawLine(n9, 0, n9, n8);
            }
            else {
                graphics.drawLine(n9, 6, n9, n8);
            }
        }
        graphics.setColor(b ? this.selectHighlight : this.highlight);
        if (!(leftToRight == 0)) {
            graphics.drawLine(1, 6, 6, 1);
            graphics.drawLine(6, 1, (tabIndex == lastTabInRun) ? (n9 - 1) : n9, 1);
            graphics.drawLine(1, 6, 1, n8);
            if (tabIndex == n7 && tabIndex != this.tabRuns[this.runCount - 1]) {
                if (this.tabPane.getSelectedIndex() == this.tabRuns[runForTab + 1]) {
                    graphics.setColor(this.selectHighlight);
                }
                else {
                    graphics.setColor(this.highlight);
                }
                graphics.drawLine(1, 0, 1, 4);
            }
        }
        else {
            graphics.drawLine(n9 - 1, 6, n9 - 6, 1);
            graphics.drawLine(n9 - 6, 1, 1, 1);
            if (tabIndex == lastTabInRun) {
                graphics.drawLine(1, 1, 1, n8);
            }
            else {
                graphics.drawLine(0, 1, 0, n8);
            }
        }
        graphics.translate(-n, -n2);
    }
    
    protected boolean shouldFillGap(int n, int i, int n2, int n3) {
        boolean n4 = false;
        if (!this.tabsOpaque) {
            return false;
        }
        if (n == this.runCount - 2) {
            Rectangle tabBounds = this.getTabBounds(this.tabPane, this.tabPane.getTabCount() - 1);
            Rectangle tabBounds2 = this.getTabBounds(this.tabPane, i);
            if (MetalUtils.isLeftToRight(this.tabPane)) {
                if (tabBounds.x + tabBounds.width - 1 > tabBounds2.x + 2) {
                    return true;
                }
            }
            else if (tabBounds.x < tabBounds2.x + tabBounds2.width - 1 - 2) {
                return true;
            }
        }
        else {
            n4 = (n != this.runCount - 1);
        }
        return n4;
    }
    
    protected Color getColorForGap(int n, int n2, int n3) {
        int selectedIndex = this.tabPane.getSelectedIndex();
        int i;
        int lastTabInRun;
        Rectangle tabBounds;
        int x;
        int n4;
        for (i = this.tabRuns[n + 1], lastTabInRun = this.lastTabInRun(this.tabPane.getTabCount(), n + 1), ; i <= lastTabInRun; ++i) {
            tabBounds = this.getTabBounds(this.tabPane, i);
            x = tabBounds.x;
            n4 = tabBounds.x + tabBounds.width - 1;
            if (MetalUtils.isLeftToRight(this.tabPane)) {
                if (x <= n2 && n4 - 4 > n2) {
                    return (selectedIndex == i) ? this.selectColor : this.getUnselectedBackgroundAt(i);
                }
            }
            else if (x + 4 < n2 && n4 >= n2) {
                return (selectedIndex == i) ? this.selectColor : this.getUnselectedBackgroundAt(i);
            }
        }
        return this.tabPane.getBackground();
    }
    
    protected void paintLeftTabBorder(int tabIndex, Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
        int tabCount = this.tabPane.getTabCount();
        int runForTab = this.getRunForTab(tabCount, tabIndex);
        int lastTabInRun = this.lastTabInRun(tabCount, runForTab);
        int n7 = this.tabRuns[runForTab];
        graphics.translate(n, n2);
        int n8 = n4 - 1;
        int n9 = n3 - 1;
        if (tabIndex != n7 && this.tabsOpaque) {
            graphics.setColor((this.tabPane.getSelectedIndex() == tabIndex - 1) ? this.selectColor : this.getUnselectedBackgroundAt(tabIndex - 1));
            graphics.fillRect(2, 0, 4, 3);
            graphics.drawLine(2, 3, 2, 3);
        }
        if (this.ocean) {
            graphics.setColor(b ? this.selectHighlight : MetalLookAndFeel.getWhite());
        }
        else {
            graphics.setColor(b ? this.selectHighlight : this.highlight);
        }
        graphics.drawLine(1, 6, 6, 1);
        graphics.drawLine(1, 6, 1, n8);
        graphics.drawLine(6, 1, n9, 1);
        if (tabIndex != n7) {
            if (this.tabPane.getSelectedIndex() == tabIndex - 1) {
                graphics.setColor(this.selectHighlight);
            }
            else {
                graphics.setColor(this.ocean ? MetalLookAndFeel.getWhite() : this.highlight);
            }
            graphics.drawLine(1, 0, 1, 4);
        }
        if (this.ocean) {
            if (b) {
                graphics.setColor(this.oceanSelectedBorderColor);
            }
            else {
                graphics.setColor(this.darkShadow);
            }
        }
        else {
            graphics.setColor(this.darkShadow);
        }
        graphics.drawLine(1, 5, 6, 0);
        graphics.drawLine(6, 0, n9, 0);
        if (tabIndex == lastTabInRun) {
            graphics.drawLine(0, n8, n9, n8);
        }
        if (this.ocean) {
            if (this.tabPane.getSelectedIndex() == tabIndex - 1) {
                graphics.drawLine(0, 5, 0, n8);
                graphics.setColor(this.oceanSelectedBorderColor);
                graphics.drawLine(0, 0, 0, 5);
            }
            else if (b) {
                graphics.drawLine(0, 6, 0, n8);
                if (tabIndex != 0) {
                    graphics.setColor(this.darkShadow);
                    graphics.drawLine(0, 0, 0, 5);
                }
            }
            else if (tabIndex != n7) {
                graphics.drawLine(0, 0, 0, n8);
            }
            else {
                graphics.drawLine(0, 6, 0, n8);
            }
        }
        else if (tabIndex != n7) {
            graphics.drawLine(0, 0, 0, n8);
        }
        else {
            graphics.drawLine(0, 6, 0, n8);
        }
        graphics.translate(-n, -n2);
    }
    
    protected void paintBottomTabBorder(int tabIndex, Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
        int tabCount = this.tabPane.getTabCount();
        int runForTab = this.getRunForTab(tabCount, tabIndex);
        int lastTabInRun = this.lastTabInRun(tabCount, runForTab);
        int n7 = this.tabRuns[runForTab];
        int leftToRight = MetalUtils.isLeftToRight(this.tabPane);
        int n8 = n4 - 1;
        int n9 = n3 - 1;
        if (this.shouldFillGap(runForTab, tabIndex, n, n2)) {
            graphics.translate(n, n2);
            if (!(leftToRight == 0)) {
                graphics.setColor(this.getColorForGap(runForTab, n, n2));
                graphics.fillRect(1, n8 - 4, 3, 5);
                graphics.fillRect(4, n8 - 1, 2, 2);
            }
            else {
                graphics.setColor(this.getColorForGap(runForTab, n + n3 - 1, n2));
                graphics.fillRect(n9 - 3, n8 - 3, 3, 4);
                graphics.fillRect(n9 - 5, n8 - 1, 2, 2);
                graphics.drawLine(n9 - 1, n8 - 4, n9 - 1, n8 - 4);
            }
            graphics.translate(-n, -n2);
        }
        graphics.translate(n, n2);
        if (this.ocean && b) {
            graphics.setColor(this.oceanSelectedBorderColor);
        }
        else {
            graphics.setColor(this.darkShadow);
        }
        if (!(leftToRight == 0)) {
            graphics.drawLine(1, n8 - 5, 6, n8);
            graphics.drawLine(6, n8, n9, n8);
            if (tabIndex == lastTabInRun) {
                graphics.drawLine(n9, 0, n9, n8);
            }
            if (this.ocean && b) {
                graphics.drawLine(0, 0, 0, n8 - 6);
                if ((runForTab == 0 && tabIndex != 0) || (runForTab > 0 && tabIndex != this.tabRuns[runForTab - 1])) {
                    graphics.setColor(this.darkShadow);
                    graphics.drawLine(0, n8 - 5, 0, n8);
                }
            }
            else {
                if (this.ocean && tabIndex == this.tabPane.getSelectedIndex() + 1) {
                    graphics.setColor(this.oceanSelectedBorderColor);
                }
                if (tabIndex != this.tabRuns[this.runCount - 1]) {
                    graphics.drawLine(0, 0, 0, n8);
                }
                else {
                    graphics.drawLine(0, 0, 0, n8 - 6);
                }
            }
        }
        else {
            graphics.drawLine(n9 - 1, n8 - 5, n9 - 6, n8);
            graphics.drawLine(n9 - 6, n8, 0, n8);
            if (tabIndex == lastTabInRun) {
                graphics.drawLine(0, 0, 0, n8);
            }
            if (this.ocean && tabIndex == this.tabPane.getSelectedIndex() + 1) {
                graphics.setColor(this.oceanSelectedBorderColor);
                graphics.drawLine(n9, 0, n9, n8);
            }
            else if (this.ocean && b) {
                graphics.drawLine(n9, 0, n9, n8 - 6);
                if (tabIndex != n7) {
                    graphics.setColor(this.darkShadow);
                    graphics.drawLine(n9, n8 - 5, n9, n8);
                }
            }
            else if (tabIndex != this.tabRuns[this.runCount - 1]) {
                graphics.drawLine(n9, 0, n9, n8);
            }
            else {
                graphics.drawLine(n9, 0, n9, n8 - 6);
            }
        }
        graphics.setColor(b ? this.selectHighlight : this.highlight);
        if (!(leftToRight == 0)) {
            graphics.drawLine(1, n8 - 6, 6, n8 - 1);
            graphics.drawLine(1, 0, 1, n8 - 6);
            if (tabIndex == n7 && tabIndex != this.tabRuns[this.runCount - 1]) {
                if (this.tabPane.getSelectedIndex() == this.tabRuns[runForTab + 1]) {
                    graphics.setColor(this.selectHighlight);
                }
                else {
                    graphics.setColor(this.highlight);
                }
                graphics.drawLine(1, n8 - 4, 1, n8);
            }
        }
        else if (tabIndex == lastTabInRun) {
            graphics.drawLine(1, 0, 1, n8 - 1);
        }
        else {
            graphics.drawLine(0, 0, 0, n8 - 1);
        }
        graphics.translate(-n, -n2);
    }
    
    protected void paintRightTabBorder(int tabIndex, Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
        int tabCount = this.tabPane.getTabCount();
        int runForTab = this.getRunForTab(tabCount, tabIndex);
        int lastTabInRun = this.lastTabInRun(tabCount, runForTab);
        int n7 = this.tabRuns[runForTab];
        graphics.translate(n, n2);
        int n8 = n4 - 1;
        int n9 = n3 - 1;
        if (tabIndex != n7 && this.tabsOpaque) {
            graphics.setColor((this.tabPane.getSelectedIndex() == tabIndex - 1) ? this.selectColor : this.getUnselectedBackgroundAt(tabIndex - 1));
            graphics.fillRect(n9 - 5, 0, 5, 3);
            graphics.fillRect(n9 - 2, 3, 2, 2);
        }
        graphics.setColor(b ? this.selectHighlight : this.highlight);
        graphics.drawLine(n9 - 6, 1, n9 - 1, 6);
        graphics.drawLine(0, 1, n9 - 6, 1);
        if (!b) {
            graphics.drawLine(0, 1, 0, n8);
        }
        if (this.ocean && b) {
            graphics.setColor(this.oceanSelectedBorderColor);
        }
        else {
            graphics.setColor(this.darkShadow);
        }
        if (tabIndex == lastTabInRun) {
            graphics.drawLine(0, n8, n9, n8);
        }
        if (this.ocean && this.tabPane.getSelectedIndex() == tabIndex - 1) {
            graphics.setColor(this.oceanSelectedBorderColor);
        }
        graphics.drawLine(n9 - 6, 0, n9, 6);
        graphics.drawLine(0, 0, n9 - 6, 0);
        if (this.ocean && b) {
            graphics.drawLine(n9, 6, n9, n8);
            if (tabIndex != n7) {
                graphics.setColor(this.darkShadow);
                graphics.drawLine(n9, 0, n9, 5);
            }
        }
        else if (this.ocean && this.tabPane.getSelectedIndex() == tabIndex - 1) {
            graphics.setColor(this.oceanSelectedBorderColor);
            graphics.drawLine(n9, 0, n9, 6);
            graphics.setColor(this.darkShadow);
            graphics.drawLine(n9, 6, n9, n8);
        }
        else if (tabIndex != n7) {
            graphics.drawLine(n9, 0, n9, n8);
        }
        else {
            graphics.drawLine(n9, 6, n9, n8);
        }
        graphics.translate(-n, -n2);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        if (component.isOpaque()) {
            graphics.setColor(this.tabAreaBackground);
            graphics.fillRect(0, 0, component.getWidth(), component.getHeight());
        }
        this.paint(graphics, component);
    }
    
    @Override
    protected void paintTabBackground(Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, boolean b) {
        if (b) {
            graphics.setColor(this.selectColor);
        }
        else {
            graphics.setColor(this.getUnselectedBackgroundAt(n2));
        }
        if (MetalUtils.isLeftToRight(this.tabPane)) {
            switch (n) {
                case 2: {
                    graphics.fillRect(n3 + 5, n4 + 1, n5 - 5, n6 - 1);
                    graphics.fillRect(n3 + 2, n4 + 4, 3, n6 - 4);
                    break;
                }
                case 3: {
                    graphics.fillRect(n3 + 2, n4, n5 - 2, n6 - 4);
                    graphics.fillRect(n3 + 5, n4 + (n6 - 1) - 3, n5 - 5, 3);
                    break;
                }
                case 4: {
                    graphics.fillRect(n3, n4 + 2, n5 - 4, n6 - 2);
                    graphics.fillRect(n3 + (n5 - 1) - 3, n4 + 5, 3, n6 - 5);
                    break;
                }
                default: {
                    graphics.fillRect(n3 + 4, n4 + 2, n5 - 1 - 3, n6 - 1 - 1);
                    graphics.fillRect(n3 + 2, n4 + 5, 2, n6 - 5);
                    break;
                }
            }
        }
        else {
            switch (n) {
                case 2: {
                    graphics.fillRect(n3 + 5, n4 + 1, n5 - 5, n6 - 1);
                    graphics.fillRect(n3 + 2, n4 + 4, 3, n6 - 4);
                    break;
                }
                case 3: {
                    graphics.fillRect(n3, n4, n5 - 5, n6 - 1);
                    graphics.fillRect(n3 + (n5 - 1) - 4, n4, 4, n6 - 5);
                    graphics.fillRect(n3 + (n5 - 1) - 4, n4 + (n6 - 1) - 4, 2, 2);
                    break;
                }
                case 4: {
                    graphics.fillRect(n3 + 1, n4 + 1, n5 - 5, n6 - 1);
                    graphics.fillRect(n3 + (n5 - 1) - 3, n4 + 5, 3, n6 - 5);
                    break;
                }
                default: {
                    graphics.fillRect(n3, n4 + 2, n5 - 1 - 3, n6 - 1 - 1);
                    graphics.fillRect(n3 + (n5 - 1) - 3, n4 + 5, 3, n6 - 3);
                    break;
                }
            }
        }
    }
    
    @Override
    protected int getTabLabelShiftX(int n, int n2, boolean b) {
        return 0;
    }
    
    @Override
    protected int getTabLabelShiftY(int n, int n2, boolean b) {
        return 0;
    }
    
    @Override
    protected int getBaselineOffset() {
        return 0;
    }
    
    @Override
    public void paint(Graphics g, JComponent c) {
        int tabPlacement = this.tabPane.getTabPlacement();
        Insets insets = c.getInsets();
        Dimension size = c.getSize();
        if (this.tabPane.isOpaque()) {
            Color background = c.getBackground();
            if (background instanceof UIResource && this.tabAreaBackground != null) {
                g.setColor(this.tabAreaBackground);
            }
            else {
                g.setColor(background);
            }
            switch (tabPlacement) {
                case 2: {
                    g.fillRect(insets.left, insets.top, this.calculateTabAreaWidth(tabPlacement, this.runCount, this.maxTabWidth), size.height - insets.bottom - insets.top);
                    break;
                }
                case 3: {
                    int calculateTabAreaHeight = this.calculateTabAreaHeight(tabPlacement, this.runCount, this.maxTabHeight);
                    g.fillRect(insets.left, size.height - insets.bottom - calculateTabAreaHeight, size.width - insets.left - insets.right, calculateTabAreaHeight);
                    break;
                }
                case 4: {
                    int calculateTabAreaWidth = this.calculateTabAreaWidth(tabPlacement, this.runCount, this.maxTabWidth);
                    g.fillRect(size.width - insets.right - calculateTabAreaWidth, insets.top, calculateTabAreaWidth, size.height - insets.top - insets.bottom);
                    break;
                }
                default: {
                    g.fillRect(insets.left, insets.top, size.width - insets.right - insets.left, this.calculateTabAreaHeight(tabPlacement, this.runCount, this.maxTabHeight));
                    this.paintHighlightBelowTab();
                    break;
                }
            }
        }
        super.paint(g, c);
    }
    
    protected void paintHighlightBelowTab() {
    }
    
    @Override
    protected void paintFocusIndicator(Graphics graphics, int n, Rectangle[] array, int n2, Rectangle rectangle, Rectangle rectangle2, boolean b) {
        if (this.tabPane.hasFocus() && b) {
            Rectangle rectangle3 = array[n2];
            int lastInRun = this.isLastInRun(n2);
            graphics.setColor(this.focus);
            graphics.translate(rectangle3.x, rectangle3.y);
            int n3 = rectangle3.width - 1;
            int n4 = rectangle3.height - 1;
            int leftToRight = MetalUtils.isLeftToRight(this.tabPane);
            switch (n) {
                case 4: {
                    graphics.drawLine(n3 - 6, 2, n3 - 2, 6);
                    graphics.drawLine(1, 2, n3 - 6, 2);
                    graphics.drawLine(n3 - 2, 6, n3 - 2, n4);
                    graphics.drawLine(1, 2, 1, n4);
                    graphics.drawLine(1, n4, n3 - 2, n4);
                    break;
                }
                case 3: {
                    if (!(leftToRight == 0)) {
                        graphics.drawLine(2, n4 - 6, 6, n4 - 2);
                        graphics.drawLine(6, n4 - 2, n3, n4 - 2);
                        graphics.drawLine(2, 0, 2, n4 - 6);
                        graphics.drawLine(2, 0, n3, 0);
                        graphics.drawLine(n3, 0, n3, n4 - 2);
                        break;
                    }
                    graphics.drawLine(n3 - 2, n4 - 6, n3 - 6, n4 - 2);
                    graphics.drawLine(n3 - 2, 0, n3 - 2, n4 - 6);
                    if (!(lastInRun == 0)) {
                        graphics.drawLine(2, n4 - 2, n3 - 6, n4 - 2);
                        graphics.drawLine(2, 0, n3 - 2, 0);
                        graphics.drawLine(2, 0, 2, n4 - 2);
                        break;
                    }
                    graphics.drawLine(1, n4 - 2, n3 - 6, n4 - 2);
                    graphics.drawLine(1, 0, n3 - 2, 0);
                    graphics.drawLine(1, 0, 1, n4 - 2);
                    break;
                }
                case 2: {
                    graphics.drawLine(2, 6, 6, 2);
                    graphics.drawLine(2, 6, 2, n4 - 1);
                    graphics.drawLine(6, 2, n3, 2);
                    graphics.drawLine(n3, 2, n3, n4 - 1);
                    graphics.drawLine(2, n4 - 1, n3, n4 - 1);
                    break;
                }
                default: {
                    if (!(leftToRight == 0)) {
                        graphics.drawLine(2, 6, 6, 2);
                        graphics.drawLine(2, 6, 2, n4 - 1);
                        graphics.drawLine(6, 2, n3, 2);
                        graphics.drawLine(n3, 2, n3, n4 - 1);
                        graphics.drawLine(2, n4 - 1, n3, n4 - 1);
                        break;
                    }
                    graphics.drawLine(n3 - 2, 6, n3 - 6, 2);
                    graphics.drawLine(n3 - 2, 6, n3 - 2, n4 - 1);
                    if (!(lastInRun == 0)) {
                        graphics.drawLine(n3 - 6, 2, 2, 2);
                        graphics.drawLine(2, 2, 2, n4 - 1);
                        graphics.drawLine(n3 - 2, n4 - 1, 2, n4 - 1);
                        break;
                    }
                    graphics.drawLine(n3 - 6, 2, 1, 2);
                    graphics.drawLine(1, 2, 1, n4 - 1);
                    graphics.drawLine(n3 - 2, n4 - 1, 1, n4 - 1);
                    break;
                }
            }
            graphics.translate(-rectangle3.x, -rectangle3.y);
        }
    }
    
    @Override
    protected void paintContentBorderTopEdge(Graphics graphics, int n, int tabIndex, int n2, int n3, int n4, int n5) {
        int leftToRight = MetalUtils.isLeftToRight(this.tabPane);
        int n6 = n2 + n4 - 1;
        Rectangle rectangle = (tabIndex < 0) ? null : this.getTabBounds(tabIndex, this.calcRect);
        if (this.ocean) {
            graphics.setColor(this.oceanSelectedBorderColor);
        }
        else {
            graphics.setColor(this.selectHighlight);
        }
        if (n != 1 || tabIndex < 0 || rectangle.y + rectangle.height + 1 < n3 || rectangle.x < n2 || rectangle.x > n2 + n4) {
            graphics.drawLine(n2, n3, n2 + n4 - 2, n3);
            if (this.ocean && n == 1) {
                graphics.setColor(MetalLookAndFeel.getWhite());
                graphics.drawLine(n2, n3 + 1, n2 + n4 - 2, n3 + 1);
            }
        }
        else {
            int lastInRun = this.isLastInRun(tabIndex);
            if (leftToRight || lastInRun) {
                graphics.drawLine(n2, n3, rectangle.x + 1, n3);
            }
            else {
                graphics.drawLine(n2, n3, rectangle.x, n3);
            }
            if (rectangle.x + rectangle.width < n6 - 1) {
                if (leftToRight && lastInRun == 0) {
                    graphics.drawLine(rectangle.x + rectangle.width, n3, n6 - 1, n3);
                }
                else {
                    graphics.drawLine(rectangle.x + rectangle.width - 1, n3, n6 - 1, n3);
                }
            }
            else {
                graphics.setColor(this.shadow);
                graphics.drawLine(n2 + n4 - 2, n3, n2 + n4 - 2, n3);
            }
            if (this.ocean) {
                graphics.setColor(MetalLookAndFeel.getWhite());
                if (leftToRight || lastInRun) {
                    graphics.drawLine(n2, n3 + 1, rectangle.x + 1, n3 + 1);
                }
                else {
                    graphics.drawLine(n2, n3 + 1, rectangle.x, n3 + 1);
                }
                if (rectangle.x + rectangle.width < n6 - 1) {
                    if (leftToRight && lastInRun == 0) {
                        graphics.drawLine(rectangle.x + rectangle.width, n3 + 1, n6 - 1, n3 + 1);
                    }
                    else {
                        graphics.drawLine(rectangle.x + rectangle.width - 1, n3 + 1, n6 - 1, n3 + 1);
                    }
                }
                else {
                    graphics.setColor(this.shadow);
                    graphics.drawLine(n2 + n4 - 2, n3 + 1, n2 + n4 - 2, n3 + 1);
                }
            }
        }
    }
    
    @Override
    protected void paintContentBorderBottomEdge(Graphics graphics, int n, int tabIndex, int n2, int n3, int n4, int n5) {
        int leftToRight = MetalUtils.isLeftToRight(this.tabPane);
        int n6 = n3 + n5 - 1;
        int n7 = n2 + n4 - 1;
        Rectangle rectangle = (tabIndex < 0) ? null : this.getTabBounds(tabIndex, this.calcRect);
        graphics.setColor(this.darkShadow);
        if (n != 3 || tabIndex < 0 || rectangle.y - 1 > n5 || rectangle.x < n2 || rectangle.x > n2 + n4) {
            if (this.ocean && n == 3) {
                graphics.setColor(this.oceanSelectedBorderColor);
            }
            graphics.drawLine(n2, n3 + n5 - 1, n2 + n4 - 1, n3 + n5 - 1);
        }
        else {
            int lastInRun = this.isLastInRun(tabIndex);
            if (this.ocean) {
                graphics.setColor(this.oceanSelectedBorderColor);
            }
            if (leftToRight || lastInRun) {
                graphics.drawLine(n2, n6, rectangle.x, n6);
            }
            else {
                graphics.drawLine(n2, n6, rectangle.x - 1, n6);
            }
            if (rectangle.x + rectangle.width < n2 + n4 - 2) {
                if (leftToRight && lastInRun == 0) {
                    graphics.drawLine(rectangle.x + rectangle.width, n6, n7, n6);
                }
                else {
                    graphics.drawLine(rectangle.x + rectangle.width - 1, n6, n7, n6);
                }
            }
        }
    }
    
    @Override
    protected void paintContentBorderLeftEdge(Graphics graphics, int n, int tabIndex, int n2, int n3, int n4, int n5) {
        Rectangle rectangle = (tabIndex < 0) ? null : this.getTabBounds(tabIndex, this.calcRect);
        if (this.ocean) {
            graphics.setColor(this.oceanSelectedBorderColor);
        }
        else {
            graphics.setColor(this.selectHighlight);
        }
        if (n != 2 || tabIndex < 0 || rectangle.x + rectangle.width + 1 < n2 || rectangle.y < n3 || rectangle.y > n3 + n5) {
            graphics.drawLine(n2, n3 + 1, n2, n3 + n5 - 2);
            if (this.ocean && n == 2) {
                graphics.setColor(MetalLookAndFeel.getWhite());
                graphics.drawLine(n2 + 1, n3, n2 + 1, n3 + n5 - 2);
            }
        }
        else {
            graphics.drawLine(n2, n3, n2, rectangle.y + 1);
            if (rectangle.y + rectangle.height < n3 + n5 - 2) {
                graphics.drawLine(n2, rectangle.y + rectangle.height + 1, n2, n3 + n5 + 2);
            }
            if (this.ocean) {
                graphics.setColor(MetalLookAndFeel.getWhite());
                graphics.drawLine(n2 + 1, n3 + 1, n2 + 1, rectangle.y + 1);
                if (rectangle.y + rectangle.height < n3 + n5 - 2) {
                    graphics.drawLine(n2 + 1, rectangle.y + rectangle.height + 1, n2 + 1, n3 + n5 + 2);
                }
            }
        }
    }
    
    @Override
    protected void paintContentBorderRightEdge(Graphics graphics, int n, int tabIndex, int n2, int n3, int n4, int n5) {
        Rectangle rectangle = (tabIndex < 0) ? null : this.getTabBounds(tabIndex, this.calcRect);
        graphics.setColor(this.darkShadow);
        if (n != 4 || tabIndex < 0 || rectangle.x - 1 > n4 || rectangle.y < n3 || rectangle.y > n3 + n5) {
            if (this.ocean && n == 4) {
                graphics.setColor(this.oceanSelectedBorderColor);
            }
            graphics.drawLine(n2 + n4 - 1, n3, n2 + n4 - 1, n3 + n5 - 1);
        }
        else {
            if (this.ocean) {
                graphics.setColor(this.oceanSelectedBorderColor);
            }
            graphics.drawLine(n2 + n4 - 1, n3, n2 + n4 - 1, rectangle.y);
            if (rectangle.y + rectangle.height < n3 + n5 - 2) {
                graphics.drawLine(n2 + n4 - 1, rectangle.y + rectangle.height, n2 + n4 - 1, n3 + n5 - 2);
            }
        }
    }
    
    @Override
    protected int calculateMaxTabHeight(int tabPlacement) {
        int height = this.getFontMetrics().getHeight();
        boolean n = false;
        int i = 0;
        while (i < this.tabPane.getTabCount()) {
            Icon icon = this.tabPane.getIconAt(i);
            if (icon != null && icon.getIconHeight() > height) {
                n = true;
                break;
            }
            ++i;
        }
        return super.calculateMaxTabHeight(tabPlacement) - (n ? (this.tabInsets.top + this.tabInsets.bottom) : 0);
    }
    
    @Override
    protected int getTabRunOverlay(int n) {
        if (n == 2 || n == 4) {
            return this.calculateMaxTabHeight(n) / 2;
        }
        return 0;
    }
    
    protected boolean shouldRotateTabRuns(int n, int n2) {
        return false;
    }
    
    @Override
    protected boolean shouldPadTabRun(int n, int n2) {
        return this.runCount > 1 && n2 < this.runCount - 1;
    }
    
    private boolean isLastInRun(int tabIndex) {
        return tabIndex == this.lastTabInRun(this.tabPane.getTabCount(), this.getRunForTab(this.tabPane.getTabCount(), tabIndex));
    }
    
    private Color getUnselectedBackgroundAt(int index) {
        Color background = this.tabPane.getBackgroundAt(index);
        if (background instanceof UIResource && this.unselectedBackground != null) {
            return this.unselectedBackground;
        }
        return background;
    }
    
    int getRolloverTabIndex() {
        return this.getRolloverTab();
    }
    
    public class TabbedPaneLayout extends BasicTabbedPaneUI.TabbedPaneLayout
    {
        @Override
        protected void normalizeTabRuns(int tabPlacement, int tabCount, int start, int max) {
            if (tabPlacement == 1 || tabPlacement == 3) {
                super.normalizeTabRuns(tabPlacement, tabCount, start, max);
            }
        }
        
        @Override
        protected void rotateTabRuns(int n, int n2) {
        }
        
        @Override
        protected void padSelectedTab(int n, int n2) {
        }
    }
}
